__description__ = 'lamuda common useful tools'
__license__ = 'MIT'
__uri__ = 'https://github.com/hanadumal/lamud'
__version__ = '21.5.6_1'
__author__ = 'hanadumal'
__email__ = 'hanadumal@outlook.com'
